package com.book.server.service.impl;


import com.book.dao.cps.mapper.*;
import com.book.dao.cps.pojo.*;
import com.book.dao.cpsshard.entity.Consume;
import com.book.dao.cpsshard.entity.Recharge;
import com.book.dao.cpsshard.entity.UserRecentlyRead;
import com.book.dao.cpsshard.mapper.ConsumeMapper;
import com.book.dao.cpsshard.mapper.RechargeMapper;
import com.book.dao.cpsshard.mapper.UserRecentlyReadMapper;
import com.book.dao.cpsshard.pojo.example.ConsumeExample;
import com.book.dao.cpsshard.pojo.example.RechargeExample;
import com.book.dao.cpsshard.pojo.example.UserRecentlyReadExample;
import com.book.server.common.entity.Result;
import com.book.server.common.entity.ResultCode;
import com.book.server.common.util.DateUtils;
import com.book.server.common.util.JsonUtils;
import com.book.dao.cps.pojo.example.*;
import com.book.dao.VO.*;
import com.book.server.service.BookService;
import com.book.server.utils.HttpTool;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class BookServiceImpl implements BookService {
    public static final Logger logger = LoggerFactory.getLogger(BookServiceImpl.class);
    @Autowired
    private BookMapper bookMapper;
    @Autowired
    private BookMapper2 bookMapper2;

    @Autowired
    private ManageBlockMapper manageBlockMapper;

    @Autowired
    private ManageBlockResourceMapper manageBlockResourceMapper;
    @Autowired
    private BookCategoryMapper bookCategoryMapper;
    @Autowired
    private SearchKeywordMapper searchKeywordMapper;
    @Autowired
    private ManageBlockResource2Mapper manageBlockResource2Mapper;

    public List<BookCategory> getAllCategory() {
        List<BookCategory> bookCategories = bookCategoryMapper.selectByExample(
                BookCategoryExample.newAndCreateCriteria().example()
        );
        return bookCategories;
    }


    @Override
    public List<BookRes> query(QueryVO queryVO) {
        queryVO.setQuery("%" + queryVO.getQuery() + "%");
        List<BookRes> bookResList = bookMapper2.queryBookRes(queryVO, queryVO.getPage() * queryVO.getSize(), queryVO.getSize());
        return bookResList;
    }

    @Override
    public List<BookRes> recommendByRead(QueryVO queryVO) {
        ArrayList<BookRes> bookRes = new ArrayList<>();
        if (queryVO.getPage() == 5) {
            return bookRes;
        }
        List<BookRes> books = bookMapper2.novelEndRecommend(queryVO,
                queryVO.getPage() * queryVO.getSize(), queryVO.getSize());
        return books;
    }

    @Override
    public List<BlockRes> getBlockByPageId(Integer pageId) {
        ArrayList<BlockRes> list = new ArrayList<>();
        // 获取板块
        ManageBlockExample manageBlockExample = ManageBlockExample.newAndCreateCriteria()
                .andPageIdEqualTo(pageId)
                .example();
        manageBlockExample.orderBy(ManageBlock.Column.weigh.desc());
        List<ManageBlock> manageBlocks = manageBlockMapper.selectByExample(manageBlockExample);


        // 获取已有版本的所有书籍
        List<Integer> blockIds = manageBlocks.stream().map(ManageBlock::getId).collect(Collectors.toList());
        List<ManageBlockResourceRes> blockResourceResList = manageBlockResource2Mapper.getBooks(blockIds);

        // 合并相同 block 书籍
        HashMap<Integer, TreeSet<ManageBlockResourceRes>> map = new HashMap<>();
        blockResourceResList.forEach(x -> {
            TreeSet<ManageBlockResourceRes> set = map.computeIfAbsent(x.getBlockId(), k -> new TreeSet<>((o1, o2) -> o2.getWeigh() - o1.getWeigh()));
            set.add(x);
        });

        manageBlocks.forEach(x -> {
            BlockRes blockRes = new BlockRes();
            blockRes.setName(x.getName());
            blockRes.setType(x.getType());
            blockRes.setSecondName(x.getSecondName());
            blockRes.setResources(map.getOrDefault(x.getId(), new TreeSet<>()));
            list.add(blockRes);
        });
        return list;
    }

    @Override
    public void insertBooks() {

        //插入书籍基本信息
        String bookInfoUrl = "http://new.emeixs.com/index.php/Home/Interface/listing";

        String jsonStr = HttpTool.sendGet(bookInfoUrl, "");
        JsonObject jsonObject = JsonUtils.getObject(jsonStr, JsonObject.class);
        JsonArray list = jsonObject.getAsJsonArray("list");

        Map<String, String> typeMap = new HashMap<String, String>();
        typeMap.put("15", "豪门总裁");
        typeMap.put("16", "婚恋家庭");
        typeMap.put("17", "豪门虐恋");
        typeMap.put("18", "甜宠萌宝");
        typeMap.put("19", "重生逆袭");
        typeMap.put("20", "穿越重生");
        typeMap.put("21", "权谋宫斗");
        typeMap.put("22", "经商种田");
        typeMap.put("23", "出版");
        typeMap.put("24", "其他");
        List<Book> bookList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            JsonElement jsonElement = list.get(i);
            JsonObject bookObject = jsonElement.getAsJsonObject();
            String author_name = bookObject.get("author_name").getAsString();
            String book_brief = bookObject.get("book_brief").getAsString();
            String book_id = bookObject.get("book_id").getAsString();
            String book_name = bookObject.get("book_name").getAsString();
            String chapter = bookObject.get("chapter").getAsString();
            String charge = bookObject.get("charge").getAsString();
            String cover = bookObject.get("cover").getAsString();
            String ctype_id = bookObject.get("ctype_id").getAsString();
            String gender = bookObject.get("gender").getAsString();
            String is_show = bookObject.get("is_show").getAsString();
            String state = bookObject.get("state").getAsString();
            String type_id = bookObject.get("type_id").getAsString();
            String words = bookObject.get("words").getAsString();

            //章节列表
            String chaptersStr = HttpTool.sendGet("http://new.emeixs.com/index.php/Home/Interface/chapters/bid/" + book_id, "");
            JsonParser parser = new JsonParser();
            JsonArray jsonArray = parser.parse(chaptersStr).getAsJsonArray();
            JsonObject firstChapters = jsonArray.get(1).getAsJsonObject();
            String first_chapter_id = firstChapters.get("content_id").getAsString();
            String first_chapter_name = firstChapters.get("name").getAsString();
            JsonObject lastChapters = jsonArray.get(jsonArray.size() - 1).getAsJsonObject();
            String last_chapter_id = lastChapters.get("content_id").getAsString();
            String last_chapter_name = lastChapters.get("name").getAsString();


            Book book = new Book();
            book.setId(Long.parseLong(book_id));
            book.setBookCategoryId(Integer.parseInt(type_id));
            book.setName(book_name);
            book.setRealname(book_name);
            book.setAuthor(author_name);
            book.setImage(cover);
            book.setDescription(book_brief);
            book.setSex(gender);
            book.setBillingType(String.valueOf(Integer.parseInt(charge) + 1));
            book.setWordCount(words);
            book.setChapterNum(Integer.valueOf(chapter));
//            book.setLastChapterUtime(Integer.parseInt(System.currentTimeMillis()+""));
            book.setCreatetime(Integer.parseInt(System.currentTimeMillis() / 1000 + ""));
            book.setCpId(Integer.parseInt(book_id));
            book.setCpName(book_name);
            book.setTags(typeMap.get(ctype_id));
            book.setState("1");
            book.setPrice(45);
            book.setArticleChapterOrder(4);
            book.setAttentChapterOrder(4);
            book.setCornerMark("hot");
            book.setIsFinish(String.valueOf(Integer.parseInt(state) - 1));
            book.setReferralNum(new Byte((byte) 5));
            book.setPutAdSet(new Byte((byte) 1));
            book.setClassifyWhiteList(new Byte((byte) 0));
            book.setCheckRank(new Byte((byte) 0));
            book.setRank(new Byte((byte) 6));
            book.setExpireTime(Integer.MAX_VALUE);
            book.setLastChapterUtime(Integer.parseInt(System.currentTimeMillis() / 1000 + ""));
            book.setAppPrice(45);
            book.setFirstChapterId(Long.parseLong(first_chapter_id));
            book.setFirstChapterName(first_chapter_name);
            book.setLastChapterId(Long.parseLong(last_chapter_id));
            book.setLastChapterName(last_chapter_name);
            book.setCansee("1");
            book.setRank(new Byte((byte) 6));

            bookList.add(book);


        }

        for (Book book : bookList) {
            int i = new Random().nextInt(bookList.size());
            book.setRecommandBookId(bookList.get(i).getId());
            System.out.println(JsonUtils.toJsonStr(book));
            bookMapper.insert(book);


        }

    }

    @Override
    public List<Chapter> getChaptersByBookId(Long bookId) {
        List<Chapter> list = new ArrayList<>();

        if (bookId == null) {
            throw new IllegalArgumentException("bookId cannot be null or empty");
        }
        final String chaptersUrl = "http://new.emeixs.com/index.php/Home/Interface/chapters/bid/" + bookId;
        String result = HttpTool.sendGet(chaptersUrl, "");
        if (StringUtils.isEmpty(result)) {
            return list;
        }
        JsonParser parser = new JsonParser();
        JsonArray jsonArray = parser.parse(result).getAsJsonArray();
        Book book = bookMapper.selectByPrimaryKey(bookId);
        if (book == null) {
            return list;
        }
        Integer freeChapterNum = book.getFreeChapterNum();

        if (freeChapterNum == null) {
            freeChapterNum = 20;
        }
        for (int i = 0; i < jsonArray.size(); i++) {
            JsonObject jsonObject = jsonArray.get(i).getAsJsonObject();
            Chapter chapter = new Chapter();
            chapter.setReadAble("1");
            chapter.setBookId(jsonObject.get("book_id").getAsString());
            chapter.setContentId(jsonObject.get("content_id").getAsString());
            chapter.setIsPay(jsonObject.get("ispay").getAsString());
            chapter.setName(jsonObject.get("name").getAsString());
            chapter.setNum(jsonObject.get("num").getAsString());
            chapter.setType(jsonObject.get("type").getAsString());
            chapter.setVolumeId(jsonObject.get("volume_id").getAsString());
            chapter.setWords(jsonObject.get("words").getAsString());
            if (i + 1 == freeChapterNum) {
                chapter.setReadAble("0");
            }
            list.add(chapter);
        }

        return list;
    }

    @Override
    public List<SearchKeyword> getSearchKeywordBySex(String sex) {
        SearchKeywordExample example = SearchKeywordExample.newAndCreateCriteria()
                .andSexEqualTo(sex)
                .example();
        example.orderBy(SearchKeyword.Column.weigh.desc());
        List<SearchKeyword> searchKeywords = searchKeywordMapper.selectByExample(example);
        return searchKeywords;
    }

    @Override
    public List<BookRes> topRecommend(QueryVO queryVO) {
        List<BookRes> bookRes = bookMapper2.topRecommend(queryVO, 0, 20);
        return getRandom(bookRes);
    }

    @Override
    public List<BookRes> smartRecommand(QueryVO queryVO) {
        List<BookRes> bookRes = bookMapper2.smartRecommand(queryVO);
        return bookRes;
    }


    @Autowired
    private ConfigMapper configMapper;

    @Autowired
    private ConsumeMapper consumeMapper;
    @Autowired
    private UserMapper userMapper;

    @Autowired
    private RechargeMapper rechargeMapper;

    @Transactional
    @Override
    public Result<BookContent> getContentByContentId(BookContentVO bookContentVO) {
        BookContent content = null;
        Book book = bookMapper.selectByPrimaryKey(bookContentVO.getBookId());
        //是否是付费章节
        Integer freeChapterNum = book.getFreeChapterNum();
        if (freeChapterNum == null) { //查看默认配置
            freeChapterNum = Integer.valueOf(configMapper.selectByName("book_free_chapter_num").getValue());
        }
        if (bookContentVO.getContentId() > freeChapterNum) { //收费
            //是否已购买
            Consume consume = consumeMapper.selectOneByExample(ConsumeExample.newAndCreateCriteria()
                    .andUserIdEqualTo(bookContentVO.getUserId())
                    .andBookIdEqualTo(bookContentVO.getBookId())
                    .andChapterIdEqualTo(bookContentVO.getContentId())
                    .example()
            );
            if (consume == null) { // 没有购买记录
                User user = userMapper.selectByPrimaryKey(bookContentVO.getUserId());
                //查询免费看点
                RechargeExample example = RechargeExample.newAndCreateCriteria()
                        .andUserIdEqualTo(bookContentVO.getUserId())
                        .andFreeEndtimeGreaterThan(DateUtils.getNow())
                        .andRemainFreeKandianGreaterThan(0)
                        .example();
                example.orderBy(Recharge.Column.freeEndtime.desc());
                List<Recharge> freeRecharges = rechargeMapper.selectByExample(example);
                Integer freeCount = 0;
                if (freeRecharges != null) {
                    freeCount = freeRecharges.stream().mapToInt(x -> x.getFreeKandian()).sum();
                }
                //免费是否足够
                if (freeCount > book.getPrice()) {
                    //扣除一部分免费的
                    int left = book.getPrice();
                    for (Recharge recharge : freeRecharges) {
                        if (left <= recharge.getRemainFreeKandian()) {
                            subtractFreeRecharge(left, recharge);
                            break;
                        } else {
                            subtractFreeRecharge(recharge.getRemainFreeKandian(), recharge);
                            left -= recharge.getRemainFreeKandian();
                        }
                    }
                    // 记录到消费表
                    addConsumer(bookContentVO, book, left, 0);
                    // 更新免费看点到 user
                    int i = userMapper.updateUserKandian(user, user.getKandian(), freeCount - book.getPrice(), DateUtils.getNow());
                    checkUpdate(i);

                } else if (user.getKandian() + freeCount > book.getPrice()) { //免费+充值足够
                    Integer left = user.getKandian() + freeCount;
                    if (freeCount != 0) {
                        //扣除所有免费
                        for (Recharge recharge : freeRecharges) {
                            subtractFreeRecharge(recharge.getRemainFreeKandian(), recharge);
                        }
                    }
                    //扣除充值的
                    int kandian = user.getKandian() - (book.getPrice() - freeCount);

                    // 记录到消费表
                    addConsumer(bookContentVO, book, freeCount, book.getPrice() - freeCount);

                    // 更新到 user
                    int i = userMapper.updateUserKandian(user, kandian, 0, DateUtils.getNow());
                    checkUpdate(i);

                } else {
                    return Result.failure(ResultCode.NOMONEY);
                }
            }

        }
        //获取章节内容
        if (bookContentVO.getBookId() > 110_0000 && bookContentVO.getBookId() < 110_9999) {
            // 自定义的书籍 TODO
        } else {
            content = getBookContentFromRemote(bookContentVO);
        }
        // 添加到阅读记录
        if (content != null) {
            //之前是否读过
            UserRecentlyRead userRecentlyRead = userRecentlyReadMapper.selectOneByExample(UserRecentlyReadExample.newAndCreateCriteria()
                    .andUserIdEqualTo(bookContentVO.getUserId())
                    .andBookIdEqualTo(bookContentVO.getBookId())
                    .example());
            if (userRecentlyRead == null) {
                int i = userRecentlyReadMapper.insert(UserRecentlyRead.builder()
                        .userId(bookContentVO.getUserId())
                        .bookId(book.getId())
                        .chapterId(bookContentVO.getContentId())
                        .chapterName(bookContentVO.getContentName())
                        .createtime(DateUtils.getNow())
                        .updatetime(DateUtils.getNow())
                        .bookShelfAdd((byte) 0)
                        .bookShelfFlag((byte) 0)
                        .build());
                //checkUpdate(i);
            } else {
                userRecentlyRead.setChapterId(bookContentVO.getContentId());
                userRecentlyRead.setChapterName(bookContentVO.getContentName());
                userRecentlyRead.setUpdatetime(DateUtils.getNow());
                int i = userRecentlyReadMapper.updateByPrimaryKeySelective(userRecentlyRead,
                        UserRecentlyRead.Column.chapterId,
                        UserRecentlyRead.Column.chapterName,
                        UserRecentlyRead.Column.updatetime
                );
                //checkUpdate(i);
            }

        }
        return Result.byObject(content);
    }

    private void checkUpdate(int i) {
        if (i <= 0) {
            throw new RuntimeException("请稍后重试");
        }
    }

    /**
     * 消费记录
     *
     * @param bookContentVO
     * @param book
     * @param freeKandian
     * @param kandian
     */
    @Transactional
    void addConsumer(BookContentVO bookContentVO, Book book, int freeKandian, int kandian) {
        int i = consumeMapper.insert(Consume.builder()
                .userId(bookContentVO.getUserId())
                .type("1") // 订购章节
                .bookId(book.getId())
                .bookName(book.getName())
                .chapterId(bookContentVO.getContentId())
                .chapterName(bookContentVO.getContentName())
                .ddKandian(kandian)
                .ddFreeKandian(freeKandian)
                .createtime(DateUtils.getNow())
                .updatetime(DateUtils.getNow())
                .businessLine((byte) 0)
                .build());
        checkUpdate(i);
    }

    /**
     * 减去多少免费看点
     *
     * @param reduceNum 消费多少免费看点
     * @param recharge
     */
    @Transactional()
    void subtractFreeRecharge(int reduceNum, Recharge recharge) {
        int i = rechargeMapper.updateByExampleSelective(
                Recharge.builder().remainFreeKandian(recharge.getRemainFreeKandian() - reduceNum).build(),
                RechargeExample.newAndCreateCriteria()
                        .andIdEqualTo(recharge.getId())
                        .andRemainFreeKandianEqualTo(recharge.getRemainFreeKandian()).example(),
                Recharge.Column.remainFreeKandian
        );
        checkUpdate(i);
    }

    /**
     * 减去多少充值看点
     *
     * @param reduceNum 消费多少看点
     * @param recharge
     */
    @Transactional
    void subtractRecharge(int reduceNum, Recharge recharge) {
        rechargeMapper.updateByExampleSelective(
                Recharge.builder().remainFreeKandian(recharge.getRemainFreeKandian() - reduceNum).build(),
                RechargeExample.newAndCreateCriteria()
                        .andIdEqualTo(recharge.getId())
                        .andRemainFreeKandianEqualTo(recharge.getRemainFreeKandian()).example(),
                Recharge.Column.remainFreeKandian
        );
    }

    /**
     * 获取版权方章节内容
     *
     * @param bookContentVO
     * @return
     */
    private BookContent getBookContentFromRemote(BookContentVO bookContentVO) {
        Long bookId = bookContentVO.getBookId();
        Long contentId = bookContentVO.getContentId();
        String contentUrl = "http://new.emeixs.com/index.php/Home/Interface/content/bid/%s/cid/%s";
        if (bookId == null || contentId == null) {
            return null;
        }
        BookContent content = null;
        try {
            content = new BookContent();
            String url = String.format(contentUrl, bookId, contentId);
            String result = HttpTool.sendGet(url, "");
            JsonObject jsonObject = JsonUtils.getObject(result, JsonObject.class);

            content.setContent(jsonObject.get("content").getAsString());
            content.setBookId(jsonObject.get("book_id").getAsString());
            content.setName(jsonObject.get("name").getAsString());
            content.setChapterId(jsonObject.get("CONTENT_ID").getAsString());
            content.setIsPay(jsonObject.get("ispay").getAsString());
        } catch (Exception e) {
            logger.info(e.getMessage());
            return null;
        }
        return content;
    }

    @Autowired
    private UserRecentlyReadMapper userRecentlyReadMapper;

    @Override
    public UserRecentlyRead getRecentReadByUserIdAndBookId(UserRecentlyRead userRecentlyRead) {
        UserRecentlyReadExample example = UserRecentlyReadExample.newAndCreateCriteria()
                .andUserIdEqualTo(userRecentlyRead.getUserId())
                .andBookIdEqualTo(userRecentlyRead.getBookId())
                .example();
        return userRecentlyReadMapper.selectOneByExample(example);
    }

    @Override
    public List<UserRecentlyRead> getRecentReadByUserId(UserRecentReadVO userRecentlyRead) {
        UserRecentlyReadExample example = UserRecentlyReadExample.newAndCreateCriteria()
                .andUserIdEqualTo(userRecentlyRead.getUserId())
                .example();
        example.orderBy(UserRecentlyRead.Column.updatetime.desc());
        example.page(userRecentlyRead.getPage(), userRecentlyRead.getSize());
        return userRecentlyReadMapper.selectByExample(example);
    }

    private List<BookRes> getRandom(List<BookRes> bookRes) {
        if (bookRes.size() <= 4) {
            return bookRes;
        }
        Collections.shuffle(bookRes);
        return bookRes.subList(0, 4);
    }


}
